@@ -57,7 +57,6 @@ extern double atan2();
 /* Declarations of global variables                                       */
 /*------------------------------------------------------------------------*/
 
-short savedir = 0;      /* for rhomboid/manhattan editing */
 char extchar[20];
 double saveratio;
 u_char texttype;
@@ -394,7 +393,7 @@ void drawdot(int xpos, int ypos)
       (*newarc)->width = 1.0;
       (*newarc)->style = FILLED | FILLSOLID | NOBORDER;
       (*newarc)->passed = NULL;
-      (*newarc)->cycle = -1;
+      (*newarc)->cycle = NULL;
       calcarc(*newarc);
       register_for_undo(XCF_Arc, UNDO_DONE, areawin->topinstance, *newarc);
    }
@@ -412,7 +411,7 @@ void labeldefaults(labelptr newlabel, u_char dopin, int x, int y)
    newlabel->scale = areawin->textscale;
    newlabel->string = (stringpart *)malloc(sizeof(stringpart));
    newlabel->passed = NULL;
-   newlabel->cycle = -1;
+   newlabel->cycle = NULL;
 
    /* initialize string with font designator */
    newlabel->string->type = FONT_NAME;
@@ -437,7 +436,7 @@ void textbutton(u_char dopin, int x, int y)
 {
    labelptr *newlabel;
    XPoint userpt;
-   short tmpheight;
+   short tmpheight, *newselect;
 
    XDefineCursor(dpy, areawin->window, TEXTPTR);
    W3printf("Click to end or cancel.");
@@ -445,8 +444,10 @@ void textbutton(u_char dopin, int x, int y)
    if (fontcount == 0)
       Wprintf("Warning:  No fonts available!");
 
+   unselect_all();
    NEW_LABEL(newlabel, topobject);
-   areawin->editpart = topobject->parts;
+   newselect = allocselect();
+   *newselect = topobject->parts;
    snap(x, y, &userpt);
    labeldefaults(*newlabel, dopin, userpt.x, userpt.y);
 
@@ -607,7 +608,7 @@ Boolean labeltext(int keypressed, char *clientdata)
 
       if (hasstuff && (eventmode != ETEXT_MODE && eventmode != CATTEXT_MODE)) {
 	 topobject->parts++;
-         register_for_undo(XCF_Text, UNDO_DONE, areawin->topinstance,
+         register_for_undo(XCF_Text, UNDO_MORE, areawin->topinstance,
                     curlabel);
 
 	 incr_changes(topobject);
@@ -615,10 +616,7 @@ Boolean labeltext(int keypressed, char *clientdata)
 
       }
       else if (!hasstuff && (eventmode == ETEXT_MODE)) {
-	 if (areawin->editpart < topobject->parts) {
-	    short *sptr = allocselect();
-	    *sptr = areawin->editpart;
-
+	 if (*(areawin->selectlist) < topobject->parts) {
 	    /* Force the "delete" undo record to be a continuation of	*/
 	    /* the undo series containing the edit.  That way, "undo"	*/
 	    /* does not generate a label with null text.		*/
@@ -631,6 +629,7 @@ Boolean labeltext(int keypressed, char *clientdata)
 	    freelabel(curlabel->string);
 	    free(curlabel);
 	    topobject->parts--;
+	    unselect_all();
 	 }
       }
 
@@ -699,7 +698,6 @@ Boolean labeltext(int keypressed, char *clientdata)
          }
 
          eventmode = CATALOG_MODE;
-	 unselect_all();
       }
       else {	/* (hasstuff && eventmode != CATTEXT_MODE) */
 	 eventmode = NORMAL_MODE;
@@ -739,6 +737,7 @@ Boolean labeltext(int keypressed, char *clientdata)
       else
          calcbbox(areawin->topinstance);
 
+      unselect_all();
       return r;
    }
    else if (clientdata != NULL && keypressed == TEXT_RIGHT) {
@@ -1060,7 +1059,7 @@ void splinedefaults(splineptr newspline, int x, int y)
    newspline->style = areawin->style;
    newspline->color = areawin->color;
    newspline->passed = NULL;
-   newspline->cycle = -1;
+   newspline->cycle = NULL;
    calcspline(newspline);
 }
 
@@ -1072,187 +1071,121 @@ void splinebutton(int x, int y)
 {
    splineptr *newspline;
    XPoint userpt;
+   short *newselect;
 
+   unselect_all();
    NEW_SPLINE(newspline, topobject);
-   areawin->editpart = topobject->parts;
+   newselect = allocselect();
+   *newselect = topobject->parts;
 
    snap(x, y, &userpt);
    splinedefaults(*newspline, userpt.x, userpt.y);
-   (*newspline)->cycle = 3;
+   addcycle((genericptr *)newspline, 3, 0);
+   makerefcycle((*newspline)->cycle, 3);
 
    XcSetXORFg(areawin->color, BACKGROUND);
    XcSetFunction(GXxor);
-   UDrawEditSpline(*newspline);
+   UDrawSpline(*newspline);
 
    xcAddEventHandler(areawin->area, PointerMotionMask, False,
-        (xcEventHandler)trackspline, NULL);
+        (xcEventHandler)trackelement, NULL);
 
    eventmode = SPLINE_MODE;
 }
 
 /*----------------------------------------------------------------------*/
-/* Find the element of a path that is currently being edited.	This	*/
-/* will be the element whose cycle is not the default -1 value.		*/
+/* Generate cycles on a path where endpoints meet, so that the path	*/
+/* remains connected during an edit.  If the last point on any part	*/
+/* of the path is a cycle, then the first point on the next part of	*/
+/* the path should also be a cycle, with the same flags.		*/
 /*----------------------------------------------------------------------*/
 
-genericptr getsubpart(pathptr editpath)
+void updatepath(pathptr thepath)
 {
-   genericptr *pgen;
-   for (pgen = editpath->plist; pgen < editpath->plist + editpath->parts; pgen++) {
-      switch (ELEMENTTYPE(*pgen)) {
-	 case POLYGON:
-	    if ((TOPOLY(pgen))->cycle >= 0) return *pgen;
-	    break;
-	 case ARC:
-	    if ((TOARC(pgen))->cycle >= 0) return *pgen;
-	    break;
-	 case SPLINE:
-	    if ((TOSPLINE(pgen))->cycle >= 0) return *pgen;
-	    break;
-      }
-   }
-   return NULL;
-}
-
-/*----------------------------------------------------------------------*/
-/* Same as the above routine, but returns a part index			*/
-/*----------------------------------------------------------------------*/
-
-int getsubpartindex(pathptr editpath)
-{
-   genericptr *pgen;
-   int ridx = 0;
-
-   for (pgen = editpath->plist; pgen < editpath->plist + editpath->parts; pgen++) {
-      switch (ELEMENTTYPE(*pgen)) {
-	 case POLYGON:
-	    if ((TOPOLY(pgen))->cycle >= 0) return ridx;
-	    break;
-	 case ARC:
-	    if ((TOARC(pgen))->cycle >= 0) return ridx;
-	    break;
-	 case SPLINE:
-	    if ((TOSPLINE(pgen))->cycle >= 0) return ridx;
-	    break;
-      }
-      ridx++;
-   }
-   return -1;
-}
-
-/*----------------------------------------------------------------------*/
-/* Adjust path points for parts connected to points that have been	*/
-/* adjusted during an edit.						*/
-/*----------------------------------------------------------------------*/
-
-void updatepath(pathptr thepath, XPoint *origpt, XPoint *newpt)
-{
-   genericptr *ggen;
+   genericptr *ggen, *ngen;
+   short locparts, cycle, ncycle;
+   pointselect *cptr;
    polyptr thispoly;
    splineptr thisspline;
-   XPoint *ppt;
 
    for (ggen = thepath->plist; ggen < thepath->plist + thepath->parts; ggen++) {
       switch (ELEMENTTYPE(*ggen)) {
 	 case POLYGON:
-	    thispoly = TOPOLY(ggen);
-	    for (ppt = thispoly->points; ppt < thispoly->points + thispoly->number;
-			ppt++) {
-	       if ((ppt->x == origpt->x) && (ppt->y == origpt->y)) {
-	          UDrawPolygon(thispoly);
-		  *ppt = *newpt;
-	          UDrawPolygon(thispoly);
-	          break; 
-	       }
-	    }
-	    break;
-
-	 case SPLINE:
-	    thisspline = TOSPLINE(ggen);
-	    if ((thisspline->ctrl[0].x == origpt->x) &&
-			(thisspline->ctrl[0].y == origpt->y)) {
-	       UDrawSpline(thisspline);
-	       thisspline->ctrl[1].x += (newpt->x - origpt->x);
-	       thisspline->ctrl[1].y += (newpt->y - origpt->y);
-	       thisspline->ctrl[0] = *newpt;
-	       calcspline(thisspline);
-	       UDrawSpline(thisspline);
-	    }
-	    else if ((thisspline->ctrl[3].x == origpt->x) &&
-			(thisspline->ctrl[3].y == origpt->y)) {
-	       UDrawSpline(thisspline);
-	       thisspline->ctrl[2].x += (newpt->x - origpt->x);
-	       thisspline->ctrl[2].y += (newpt->y - origpt->y);
-	       thisspline->ctrl[3] = *newpt;
-	       calcspline(thisspline);
-	       UDrawSpline(thisspline);
-	    }
+	    findconstrained(TOPOLY(ggen));
 	    break;
       }
    }
-}
 
-/*------------------------------------*/
-/* Track a spline during mouse motion */
-/*------------------------------------*/
-
-void trackspline(xcWidget w, caddr_t clientdata, caddr_t calldata)
-{
-   XPoint       origpt, newpos;
-   splineptr    newspline;
-
-   if (eventmode == EPATH_MODE) {
-      newspline = (splineptr)getsubpart(TOPATH(EDITPART));
+   locparts = (thepath->style & UNCLOSED) ? thepath->parts - 1 : thepath->parts;
+   for (ggen = thepath->plist; ggen < thepath->plist + locparts; ggen++) {
+      ngen = (ggen == thepath->plist + thepath->parts - 1) ? thepath->plist : ggen + 1;
+    
+      switch (ELEMENTTYPE(*ggen)) {
+	 case POLYGON:
+	    thispoly = TOPOLY(ggen);
+	    if (thispoly->cycle == NULL) continue;
+	    cycle = thispoly->number - 1;
+	    for (cptr = thispoly->cycle;; cptr++) {
+	       if (cptr->number == cycle) break;
+	       if (cptr->flags & LASTENTRY) break;
+	    }
+	    if (cptr->number != cycle) continue;
+	    break;
+	 case SPLINE:
+	    thisspline = TOSPLINE(ggen);
+	    if (thisspline->cycle == NULL) continue;
+	    cycle = 3;
+	    for (cptr = thisspline->cycle;; cptr++) {
+	       if (cptr->number == cycle) break;
+	       if (cptr->flags & LASTENTRY) break;
+	    }
+	    if (cptr->number != cycle) continue;
+	    break;
+      }
+      addcycle(ngen, 0, cptr->flags & (EDITX | EDITY));
+      switch (ELEMENTTYPE(*ngen)) {
+	 case POLYGON:
+	    findconstrained(TOPOLY(ngen));
+	    break;
+      }
    }
-   else {
-      newspline = TOSPLINE(EDITPART);
-   }
-   newpos = UGetCursorPos();
-   u2u_snap(&newpos);
-   if (areawin->save.x == newpos.x && areawin->save.y == newpos.y) return;
 
-   UDrawEditSpline(newspline);
-
-   if (newspline->cycle == 0 || newspline->cycle == 3) {
-      short cpoint = (newspline->cycle == 0) ? 1 : 2;
-      newspline->ctrl[cpoint].x += (newpos.x - newspline->ctrl[newspline->cycle].x);
-      newspline->ctrl[cpoint].y += (newpos.y - newspline->ctrl[newspline->cycle].y);
-   }
-   origpt = newspline->ctrl[newspline->cycle];
-   newspline->ctrl[newspline->cycle] = newpos;
-
-   /* In case a constrained (parameterized) point has been changed, re-substitute */
-   exprsub(eventmode == EPATH_MODE ? TOGENERIC(EDITPART) : (genericptr)newspline);
-
-   calcspline(newspline);
-   UDrawEditSpline(newspline);
-
-   printpos(newpos.x, newpos.y);
-   areawin->save.x = newpos.x;
-   areawin->save.y = newpos.y;
-
-   if (eventmode == EPATH_MODE)
-      if (newspline->cycle == 0 || newspline->cycle == 3)
-         updatepath(TOPATH(EDITPART), &origpt, &newpos);
-
-   flusharea();
-}
-
-/*--------------------------------------*/
-/* Track a path during mouse motion	*/
-/*--------------------------------------*/
-
-void trackpath(xcWidget w, caddr_t clientdata, caddr_t calldata)
-{
-   genericptr ggen = getsubpart(TOPATH(EDITPART));
-
-   switch (ELEMENTTYPE(ggen)) {
-      case POLYGON:
-	 trackpoly(w, clientdata, calldata);
-	 break;
-      case SPLINE:
-	 trackspline(w, clientdata, calldata);
-	 break;
+   /* Do the same thing in the other direction */
+   locparts = (thepath->style & UNCLOSED) ? 1 : 0;
+   for (ggen = thepath->plist + thepath->parts - 1; ggen >= thepath->plist + locparts;
+		ggen--) {
+      ngen = (ggen == thepath->plist) ? thepath->plist + thepath->parts - 1 : ggen - 1;
+    
+      switch (ELEMENTTYPE(*ggen)) {
+	 case POLYGON:
+	    thispoly = TOPOLY(ggen);
+	    if (thispoly->cycle == NULL) continue;
+	    cycle = 0;
+	    for (cptr = thispoly->cycle;; cptr++) {
+	       if (cptr->number == cycle) break;
+	       if (cptr->flags & LASTENTRY) break;
+	    }
+	    if (cptr->number != cycle) continue;
+	    break;
+	 case SPLINE:
+	    thisspline = TOSPLINE(ggen);
+	    if (thisspline->cycle == NULL) continue;
+	    cycle = 0;
+	    for (cptr = thisspline->cycle;; cptr++) {
+	       if (cptr->number == cycle) break;
+	       if (cptr->flags & LASTENTRY) break;
+	    }
+	    if (cptr->number != cycle) continue;
+	    break;
+      }
+      switch (ELEMENTTYPE(*ngen)) {
+	 case POLYGON:
+            addcycle(ngen, TOPOLY(ngen)->number - 1, cptr->flags & (EDITX | EDITY));
+	    break;
+	 case SPLINE:
+            addcycle(ngen, 3, cptr->flags & (EDITX | EDITY));
+	    break;
+      }
    }
 }
 
@@ -1272,7 +1205,7 @@ void arcdefaults(arcptr newarc, int x, int y)
    newarc->angle1 = 0;
    newarc->angle2 = 360;
    newarc->passed = NULL;
-   newarc->cycle = -1;
+   newarc->cycle = NULL;
    calcarc(newarc);
 }
 
@@ -1284,13 +1217,16 @@ void arcbutton(int x, int y)
 {
    arcptr *newarc;
    XPoint userpt;
+   short *newselect;
 
+   unselect_all();
    NEW_ARC(newarc, topobject);
-   areawin->editpart = topobject->parts;
+   newselect = allocselect();
+   *newselect = topobject->parts;
    snap(x, y, &userpt);
    saveratio = 1.0;
    arcdefaults(*newarc, userpt.x, userpt.y);
-   (*newarc)->cycle = 0;
+   addcycle((genericptr *)newarc, 0, 0);
 
    XcSetXORFg(areawin->color, BACKGROUND);
    XcSetFunction(GXxor);
@@ -1312,6 +1248,7 @@ void trackarc(xcWidget w, caddr_t clientdata, caddr_t calldata)
    XPoint newpos;
    arcptr newarc;
    double adjrat;
+   short  cycle;
 
    newarc = TOARC(EDITPART);
 
@@ -1322,15 +1259,16 @@ void trackarc(xcWidget w, caddr_t clientdata, caddr_t calldata)
    UDrawArc(newarc);
    UDrawXLine(areawin->save, newarc->position);
 
-   if (newarc->cycle == 1 || newarc->cycle == 2) {
+   cycle = (newarc->cycle == NULL) ? -1 : newarc->cycle->number;
+   if (cycle == 1 || cycle == 2) {
       float *angleptr, tmpang;
 
       adjrat = (newarc->yaxis == 0) ? 1 :
 		(double)(abs(newarc->radius)) / (double)newarc->yaxis;
-      angleptr = (newarc->cycle == 1) ? &newarc->angle1 : &newarc->angle2;
+      angleptr = (cycle == 1) ? &newarc->angle1 : &newarc->angle2;
       tmpang = (float)(atan2((double)(newpos.y - newarc->position.y) * adjrat,
 	   (double)(newpos.x - newarc->position.x)) / RADFAC);
-      if (newarc->cycle == 1) {
+      if (cycle == 1) {
 	 if (tmpang > newarc->angle2) tmpang -= 360;
 	 else if (newarc->angle2 - tmpang > 360) newarc->angle2 -= 360;
       }
@@ -1347,7 +1285,7 @@ void trackarc(xcWidget w, caddr_t clientdata, caddr_t calldata)
       if (newarc->angle2 <= newarc->angle1)
 	 newarc->angle1 -= 360;
    }
-   else if (newarc->cycle == 0) {
+   else if (cycle == 0) {
       short direc = (newarc->radius < 0);
       newarc->radius = wirelength(&newpos, &(newarc->position));
       newarc->yaxis = (short)((double)newarc->radius * saveratio);
@@ -1383,7 +1321,7 @@ void polydefaults(polyptr newpoly, int number, int x, int y)
    newpoly->width = areawin->linewidth;
    newpoly->number = number;
    newpoly->passed = NULL;
-   newpoly->cycle = -1;
+   newpoly->cycle = NULL;
    if (number == 0)
       newpoly->points = NULL;
    else {
@@ -1405,9 +1343,12 @@ void boxbutton(int x, int y)
 {
    polyptr *newbox;
    XPoint userpt;
+   short *newselect;
 
+   unselect_all();
    NEW_POLY(newbox, topobject);
-   areawin->editpart = topobject->parts;
+   newselect = allocselect();
+   *newselect = topobject->parts;
    snap(x, y, &userpt);
    polydefaults(*newbox, 4, userpt.x, userpt.y);
 
@@ -1503,9 +1444,12 @@ void startwire(XPoint *userpt)
 {
    polyptr *newwire;
    pointlist pointptr;
+   short *newselect;
 
+   unselect_all();
    NEW_POLY(newwire, topobject);
-   areawin->editpart = topobject->parts;
+   newselect = allocselect();
+   *newselect = topobject->parts;
 
    /* always start unfilled, unclosed; can fix on next button-push. */
 
@@ -1515,7 +1459,7 @@ void startwire(XPoint *userpt)
    (*newwire)->width = areawin->linewidth;
    (*newwire)->points = (pointlist) malloc(2 * sizeof(XPoint));
    (*newwire)->passed = NULL;
-   (*newwire)->cycle = -1;
+   (*newwire)->cycle = NULL;
    pointptr = (*newwire)->points;
    pointptr->x = (pointptr + 1)->x = areawin->save.x = userpt->x;
    pointptr->y = (pointptr + 1)->y = areawin->save.y = userpt->y;
@@ -1528,152 +1472,136 @@ void startwire(XPoint *userpt)
           (xcEventHandler)trackwire, NULL);
 }
 
-/*----------------------------------------------------------*/
-/* Find which points should track along with the edit point */
-/* in polygon RHOMBOID or MANHATTAN edit modes.		    */
-/* (point number is stored in lastpoly->cycle)	    	    */
-/*----------------------------------------------------------*/
+/*--------------------------------------------------------------*/
+/* Find which points should track along with the edit point in	*/
+/* in polygon RHOMBOID or MANHATTAN edit modes.		    	*/
+/* (point number is stored in lastpoly->cycle)	    	    	*/
+/*								*/
+/* NOTE:  This routine assumes that either the points have just	*/
+/* been selected, or that advancecycle() has been called to	*/
+/* remove all previously recorded tracking points.		*/
+/*--------------------------------------------------------------*/
 
-void finddir(polyptr lastpoly)
+void findconstrained(polyptr lastpoly)
 {
    XPoint *savept, *npt, *lpt;
+   short cycle;
+   short lflags, nflags;
+   short lcyc, ncyc;
+   pointselect *cptr, *nptr;
 
-   savedir = NONE;
    if (areawin->boxedit == NORMAL) return;
 
-   savept = lastpoly->points + lastpoly->cycle;
+   if (lastpoly->cycle == NULL) return;
 
-   /* find points before and after the edit point */      
-
-   lpt = (lastpoly->cycle == 0) ? ((lastpoly->style & UNCLOSED) ?
-	 NULL : lastpoly->points + lastpoly->number - 1) : savept - 1;
-   npt = (lastpoly->cycle == lastpoly->number - 1) ?
-	 ((lastpoly->style & UNCLOSED) ? NULL : lastpoly->points) :
-	 savept + 1;
-
-   /* two-point polygons (lines) are a degenerate case in RHOMBOID edit mode */
-
-   if (areawin->boxedit != MANHATTAN && lastpoly->number <= 2) return;
-
-   /* This is complicated but logical:  in MANHATTAN mode, boxes maintain */
-   /* box shape.  In RHOMBOID modes, parallelagrams maintain shape.  The  */
-   /* "savedir" variable determines which coordinate(s) of which point(s) */
-   /* should track along with the edit point.				  */
-
-   if (areawin->boxedit != RHOMBOIDY) {
-      if (lpt != NULL) {
-         if (lpt->y == savept->y) {
-	    savedir |= LASTY;
-	    if (areawin->boxedit == RHOMBOIDX && lpt->x != savept->x)
-	       savedir |= LASTX;
-	    else if (areawin->boxedit == RHOMBOIDA && npt != NULL) {
-	       if (npt->y != savept->y) savedir |= NEXTX;
-	    }
-	 }
-      }
-      if (npt != NULL) {
-         if (npt->y == savept->y) {
-	    savedir |= NEXTY;
-	    if (areawin->boxedit == RHOMBOIDX && npt->x != savept->x)
-	       savedir |= NEXTX;
-	    else if (areawin->boxedit == RHOMBOIDA && lpt != NULL) {
-	       if (lpt->y != savept->y) savedir |= LASTX;
-	    }
-	 }
-      }
+   /* Set "process" flags on all original points */
+   for (cptr = lastpoly->cycle;; cptr++) {
+      cptr->flags |= PROCESS;
+      if (cptr->flags & LASTENTRY) break;
    }
-   if (areawin->boxedit != RHOMBOIDX) {
-      if (lpt != NULL) {
-         if (lpt->x == savept->x) {
-	    savedir |= LASTX;
-	    if (areawin->boxedit == RHOMBOIDY && lpt->y != savept->y)
-	       savedir |= LASTY;
-	    else if (areawin->boxedit == RHOMBOIDA && npt != NULL) {
-	       if (npt->x != savept->x) savedir |= NEXTY;
-	    } 
-	 }
-      }
-      if (npt != NULL) {
-         if (npt->x == savept->x) {
-	    savedir |= NEXTX;
-	    if (areawin->boxedit == RHOMBOIDY && npt->y != savept->y)
-	       savedir |= NEXTY;
-	    else if (areawin->boxedit == RHOMBOIDA && lpt != NULL) {
-	       if (lpt->x != savept->x) savedir |= LASTY;
+
+   cptr = lastpoly->cycle;
+   while (1) {
+      if (cptr->flags & PROCESS) {
+	 cptr->flags &= ~PROCESS;
+         cycle = cptr->number;
+         savept = lastpoly->points + cycle;
+
+         /* find points before and after the edit point */      
+
+         lcyc = (cycle == 0) ? ((lastpoly->style & UNCLOSED) ?
+		-1 : lastpoly->number - 1) : cycle - 1;
+         ncyc = (cycle == lastpoly->number - 1) ?
+		((lastpoly->style & UNCLOSED) ? -1 : 0) : cycle + 1;
+    
+         lpt = (lcyc == -1) ? NULL : lastpoly->points + lcyc;
+         npt = (ncyc == -1) ? NULL : lastpoly->points + ncyc;
+
+	 lflags = nflags = NONE;
+
+         /* two-point polygons (lines) are a degenerate case in RHOMBOID edit mode */
+
+         if (areawin->boxedit != MANHATTAN && lastpoly->number <= 2) return;
+
+         /* This is complicated but logical:  in MANHATTAN mode, boxes maintain */
+         /* box shape.  In RHOMBOID modes, parallelagrams maintain shape.  The  */
+         /* "savedir" variable determines which coordinate(s) of which point(s) */
+         /* should track along with the edit point.			     */
+
+         if (areawin->boxedit != RHOMBOIDY) {
+            if (lpt != NULL) {
+               if (lpt->y == savept->y) {
+	          lflags |= EDITY;
+	          if (areawin->boxedit == RHOMBOIDX && lpt->x != savept->x)
+		     lflags |= EDITX;
+	          else if (areawin->boxedit == RHOMBOIDA && npt != NULL) {
+	             if (npt->y != savept->y) nflags |= EDITX;
+		  }
+	       }
 	    }
+            if (npt != NULL) {
+               if (npt->y == savept->y) {
+	          nflags |= EDITY;
+	          if (areawin->boxedit == RHOMBOIDX && npt->x != savept->x)
+		     nflags |= EDITX;
+	          else if (areawin->boxedit == RHOMBOIDA && lpt != NULL) {
+	             if (lpt->y != savept->y) lflags |= EDITX;
+	          }
+	       }
+	    }
+         }
+         if (areawin->boxedit != RHOMBOIDX) {
+            if (lpt != NULL) {
+               if (lpt->x == savept->x) {
+	          lflags |= EDITX;
+	          if (areawin->boxedit == RHOMBOIDY && lpt->y != savept->y)
+		     lflags |= EDITY;
+	          else if (areawin->boxedit == RHOMBOIDA && npt != NULL) {
+	             if (npt->x != savept->x) nflags |= EDITY;
+	          }
+	       } 
+	    }
+            if (npt != NULL) {
+               if (npt->x == savept->x) {
+	          nflags |= EDITX;
+	          if (areawin->boxedit == RHOMBOIDY && npt->y != savept->y)
+		     nflags |= EDITY;
+	          else if (areawin->boxedit == RHOMBOIDA && lpt != NULL) {
+	             if (lpt->x != savept->x) lflags |= EDITY;
+	          }
+	       }
+	    }
+         }
+	 nptr = cptr + 1;
+         if (lpt != NULL && lflags != 0) {
+	    addcycle((genericptr *)(&lastpoly), lcyc, lflags);
+	    cptr = nptr = lastpoly->cycle;
+	 }
+         if (npt != NULL && nflags != 0) {
+	    addcycle((genericptr *)(&lastpoly), ncyc, nflags);
+	    cptr = nptr = lastpoly->cycle;
 	 }
       }
+      else
+	 nptr = cptr + 1;
+      if (cptr->flags & LASTENTRY) break;
+      cptr = nptr;
    }
 }
 
-/*--------------------------------------------------------------*/
-/* Edit polygon points according to cursor position (newpos)	*/ 
-/* and polygon edit style (Manhattan, Rhomboid, etc.).	The	*/
-/* original point position is returned in origpt.		*/
-/*--------------------------------------------------------------*/
+/*------------------------------------------------------*/
+/* Track movement of poly, spline, or polygon segments	*/
+/* during edit mode					*/
+/*------------------------------------------------------*/
 
-void boxeditadjust(polyptr newpoly, XPoint *newpos, XPoint *origpt)
+void trackelement(xcWidget w, caddr_t clientdata, caddr_t calldata)
 {
-   XPoint *curpt, *fpt, *bpt;
-   int deltax, deltay;
-
-   /* find the point under consideration */
-   curpt = newpoly->points + newpoly->cycle;
-
-   if (areawin->boxedit != NORMAL) {
-
-      /* find points to either side of the edit point */
-
-      fpt = bpt = NULL;
-      deltax = newpos->x - curpt->x;
-      deltay = newpos->y - curpt->y; 
-
-      if (curpt > newpoly->points) bpt = curpt - 1;
-      else if (!(newpoly->style & UNCLOSED))
-	 bpt = newpoly->points + newpoly->number - 1;
-      if (curpt < newpoly->points + newpoly->number - 1) fpt = curpt + 1;
-      else if (!(newpoly->style & UNCLOSED)) fpt = newpoly->points;
-
-      /* enforce constraints */
-
-      if (bpt != NULL) {
-	 *origpt = *bpt;
-	 if (savedir & LASTX) bpt->x += deltax;
-	 if (savedir & LASTY) bpt->y += deltay;
-
-	 if ((eventmode == EPATH_MODE) && (savedir & (LASTX | LASTY)))
-	    if (newpoly->cycle == 1)
-	       updatepath(TOPATH(EDITPART), origpt, bpt);
-      }
-      if (fpt != NULL) {
-	 *origpt = *fpt;
-	 if (savedir & NEXTX) fpt->x += deltax;
-	 if (savedir & NEXTY) fpt->y += deltay;
-
-	 if ((eventmode == EPATH_MODE) && (savedir & (NEXTX | NEXTY)))
-	    if (newpoly->cycle == newpoly->number - 2)
-	       updatepath(TOPATH(EDITPART), origpt, fpt);
-      }
-   }
-
-   /* update position of the point under consideration */
-
-   *origpt = *curpt;
-   *curpt = *newpos;
-
-   /* In case a constrained (parameterized) point has been changed, re-substitute */
-   exprsub(eventmode == EPATH_MODE ? TOGENERIC(EDITPART) : (genericptr)newpoly);
-}
-
-/*--------------------------------------------------*/
-/* Track movement of poly segments during edit mode */
-/*--------------------------------------------------*/
-
-void trackpoly(xcWidget w, caddr_t clientdata, caddr_t calldata)
-{
-   XPoint newpos, origpt;
-   polyptr      newpoly;
+   XPoint newpos, origpt, *curpt;
+   polyptr      editpoly;
    genericptr	*ggen;
+   short	cycle, *selobj;
+   pointselect	*cptr;
+   int		deltax, deltay;
 
    newpos = UGetCursorPos();
    u2u_snap(&newpos);
@@ -1687,42 +1615,46 @@ void trackpoly(xcWidget w, caddr_t clientdata, caddr_t calldata)
 
    if (areawin->save.x == newpos.x && areawin->save.y == newpos.y) return;
 
-   if (eventmode == EPATH_MODE) 
-      newpoly = (polyptr)getsubpart(TOPATH(EDITPART));
-   else
-      newpoly = TOPOLY(EDITPART);
+   /* Find the reference point */
 
-   UDrawPolygon(newpoly);
-   boxeditadjust(newpoly, &newpos, &origpt);
-   UDrawPolygon(newpoly);
-
-   if (areawin->multiedit) {
-      polyptr tpoly;
-      XPoint torig, dpos, *curpt, *tpt;
-
-      curpt = newpoly->points + newpoly->cycle;
-      for (ggen = topobject->plist; ggen < topobject->plist + topobject->parts;
-		ggen++) {
-	 if (ELEMENTTYPE(*ggen) != POLYGON) continue;
-	 tpoly = TOPOLY(ggen);
-	 if (tpoly->cycle < 0) continue;
-	 if (tpoly == newpoly) continue;
-         tpt = tpoly->points + tpoly->cycle;
-	 dpos.x = tpt->x + (curpt->x - origpt.x);
-	 dpos.y = tpt->y + (curpt->y - origpt.y);
-         UDrawPolygon(tpoly);
-         boxeditadjust(tpoly, &dpos, &torig);
-         UDrawPolygon(tpoly);
-      }
+   cptr = getrefpoint(TOGENERIC(EDITPART), &curpt);
+   switch(ELEMENTTYPE(TOGENERIC(EDITPART))) {
+      case POLYGON:
+         if (cptr == NULL)
+	    curpt = TOPOLY(EDITPART)->points;
+	 break;
+      case SPLINE:
+         if (cptr == NULL)
+	    curpt = &(TOSPLINE(EDITPART)->ctrl[0]);
+	 break;
+      case ARC:
+	 curpt = &(TOARC(EDITPART)->position);
+	 break;
+      case OBJINST:
+	 curpt = &(TOOBJINST(EDITPART)->position);
+	 break;
+      case GRAPHIC:
+	 curpt = &(TOGRAPHIC(EDITPART)->position);
+	 break;
    }
+   origpt = *curpt;
+   deltax = newpos.x - curpt->x;
+   deltay = newpos.y - curpt->y;
+
+   /* Now adjust all edited elements relative to the reference point */
+
+   for (selobj = areawin->selectlist; selobj < areawin->selectlist +
+		areawin->selects; selobj++)
+   {
+      easydraw(*selobj, DOFORALL);
+      editpoints(SELTOGENERICPTR(selobj), deltax, deltay);
+      easydraw(*selobj, DOFORALL);
+   }
+
    printpos(newpos.x, newpos.y);
    areawin->save.x = newpos.x;
    areawin->save.y = newpos.y;
 
-   if (eventmode == EPATH_MODE)
-      if (newpoly->cycle == 0 || newpoly->cycle == newpoly->number - 1)
-         updatepath(TOPATH(EDITPART), &origpt, &newpos);
-
    flusharea();
 }
 
@@ -2100,7 +2032,7 @@ void join()
       (*newpoly)->style  = polytype;
       (*newpoly)->color  = polycolor;
       (*newpoly)->passed = NULL;
-      (*newpoly)->cycle = -1;
+      (*newpoly)->cycle = NULL;
 
       /* insert the points into the new polygon */
 
@@ -2307,9 +2239,10 @@ void wire_op(int op, int x, int y)
 	 newwire->style = areawin->style;
       poly_add_point(newwire, &userpt);
    }
-   else if ((newwire == NULL) || op == XCF_Finish_Element || op == XCF_Cancel)
+   else if ((newwire == NULL) || op == XCF_Finish_Element || op == XCF_Cancel) {
       xcRemoveEventHandler(areawin->area, PointerMotionMask, False,
          (xcEventHandler)trackwire, NULL);
+   }
 
    if (newwire) {
       if (op == XCF_Finish_Element) {
@@ -2329,11 +2262,11 @@ void wire_op(int op, int x, int y)
 	 }
 
          XcSetFunction(GXcopy);
-         XcSetForeground(newwire->color);
+         XcTopSetForeground(newwire->color);
 	 topobject->parts++;
 	 incr_changes(topobject);
 	 if (!nonnetwork(newwire)) invalidate_netlist(topobject);
-	 register_for_undo(XCF_Wire, UNDO_DONE, areawin->topinstance, newwire);
+	 register_for_undo(XCF_Wire, UNDO_MORE, areawin->topinstance, newwire);
       }
       UDrawPolygon(newwire);
       if (op == XCF_Cancel_Last)
